/**
 * Extracts the parameters of a Howland source from an output impedance spectrum.
 *
 * Usage: ./extract < impedance_spectrum_file
 */

#include <cstdio>
#include <complex>
#include <vector>
#include "opamp.cpp"
#include "data.cpp"
#include "howland.cpp"

extern "C" {
    #include <cpso.h>
}

using namespace std;

Data data;
const double cost_weight = 1e-7; // higher value tries to save more resources

static double fitness(CpsoState* st, const double* v) {
    Howland h( OpAmp(v), 100, 1);
    return -(h.mse(data) + h.cost() * cost_weight);
}

#include <sys/time.h>
unsigned long random_seed() {
    struct timeval tv;
    gettimeofday(&tv, 0);
    return tv.tv_usec + tv.tv_sec;
}

int main(int argc, char* argv[]) {
    /* load data */
    data.load_file(stdin);

    /* configure the PSO algorithm */
    CpsoState* st = cpso_new(5, 40);

    cpso_set_limits(st, 0, 1    , 100); // Ri
    cpso_set_limits(st, 1, 0    , 2  ); // Ci
    cpso_set_limits(st, 2, 5    , 100); // Ro
    cpso_set_limits(st, 3, 70   , 120); // Ao
    cpso_set_limits(st, 4, 0.001, 1  ); // Wc

    srand( random_seed() );
    cpso_set_eval(st, fitness, 0);
    cpso_set_params(st, 1.85, 1.5, 0.55, 1e-3, 1e-5);

    /* optimize */
    cpso_init(st);
    cpso_run(st, 2000, 0);

    /* output the best solution */
    const double* best;
    double cost = cpso_get_best(st, -1, &best);

    Howland h( OpAmp(best), 100, 1);
    double fit = h.mse(data);

    fprintf(stderr, "err (with cost): %g\n", -cost);
    fprintf(stderr, "err (fit only ): %g\n", fit);
    fprintf(stderr, "params:\n");
    for(int i = 0; i < 5; ++i)
        fprintf(stderr, "%12.4lf\n", best[i]);

    h.freq_sweep(data);

    return 0;
}
